//
// sas@elportal.pl
//
// Dodatkowe komunikaty w serial monitorze 960 8N1
//


#include  "EdWws2812.h"


#define L_LED   320         // Liczba LED. Max 640 (wymaga 1920 bajtów w trybie 24-bit, 1280 w 16-bit, 640 dla 8-bit tylko na sam bufor LED.
byte maxKrok = 80;        // Max jasność w trybie 24 i 16-bit
#define JASNOSC_LED   0x60      // Max jasność w trybie w trybach 8-bit (ważne 6 najstarszych bitów)

#define I2C             // Zadeklarowanie włącza komunikację po I2C, brak deklaracji SPI
#define ADRES_SLAVE   0 // Adres Slave (0..8) dla I2C i SPI. Musi być zgodny z adresem ustawionym na płytce konwertera
#define SPEED_I2C   400e3 // Zalecane 400kHz, podciąganie 2,2k, 500 LED 25ms
#define SPEED_SPI   1e6 // Zalecane 1MHz, 500 LED 9,6ms


byte cfgWs2812 = WS2812_CFG_GAMMA | ADRES_SLAVE | WS2812_CFG_COLOR_16;  // Kolor 16-bit R5 G6 B5
//byte cfgWs2812 = WS2812_CFG_GAMMA | ADRES_SLAVE;  // 24-bit
//byte cfgWs2812 = WS2812_CFG_GAMMA | ADRES_SLAVE | WS2812_CFG_COLOR_218; // Bezpieczna paleta 218 barw + 16 (VT100)
//byte cfgWs2812 = WS2812_CFG_GAMMA | ADRES_SLAVE | WS2812_CFG_COLOR_8;   // Kolor 8-bit R3 G3 B2


//================================================================//
//EdWws2812 EdWws2812( L_LED, cfgWs2812, 10, 2e6 );
// Parametry funkcji:
// - Liczba led,
// - konfiguracja,
// - pin CS (dla trybu SPI, jeśli 0 to tryb I2C),
// - szybkość transmisji (dla SPI 8, 4, 2, 1MHz, 500, 250, 125kHz), dla I2C max 800kHz
#ifndef I2C
EdWws2812 EdWws2812( L_LED, cfgWs2812, 10, SPEED_SPI );
#else
EdWws2812 EdWws2812( L_LED, cfgWs2812, 0, SPEED_I2C );
#endif


#include <Wire.h>
#include <avr/boot.h>


void skanujI2c() {
  Wire.begin();
  byte error, nDevices = 0;
  Serial.println("I2C scaning ");
  for (byte address = WS2812_BASE_I2C >> 1; address < (WS2812_BASE_I2C >> 1) + 8; address++ ) {
    // The i2c_scanner uses the return value of
    // the Write.endTransmisstion to see if
    // a device did acknowledge to the address.
    Wire.beginTransmission(address);
    error = Wire.endTransmission();

    if (error == 0) {
      Serial.print("I2C device found at address 0x");
      if (address < 16) Serial.print("0");
      Serial.print(address, HEX);
      Serial.println("  !");

      nDevices++;
    }
    else if (error == 4) {
      Serial.print("Unknown error at address 0x");
      if (address < 16) Serial.print("0");
      Serial.println(address, HEX);
    }
    else {
      if ( ! (address % 16 ) ) Serial.println(".");
      else Serial.print(".");
    }
  }
  if (nDevices == 0)
    Serial.println("No I2C devices found\n");
  else
    Serial.println("\n\rdone");
}


//================================================================//
void setup() {
  Serial.begin(9600);
  pinMode(9, OUTPUT);
  digitalWrite( 10, HIGH );
  pinMode(10, OUTPUT);
  Serial.println("*** RESET ***");


  if ( (cfgWs2812 & WS2812_CFG_COLOR_8) == WS2812_CFG_COLOR_8 ) maxKrok = 255;


  char txt[150];

  byte f0, f2, f4;
  f0 = boot_signature_byte_get( 0 );
  f2 = boot_signature_byte_get( 2 );
  f4 = boot_signature_byte_get( 4 );
  sprintf_P( txt, PSTR("CPU ID [%02x %02x %02x]"), f0, f2, f4 ); Serial.println( txt );


  int ram = 2048; // dla UNO
  if( f2 == 0x95 ) { Serial.println( "2kB RAM" ); ram = 2048; } //todo: pełniejsze informacje w bajcie f4
  else if( f2==0x98 ){ Serial.println( "8kB RAM" ); ram = 8192; } //todo: rozroznienie pomiędzy 4, 8 16kB na podstawie w bajc f4

  byte *buf;
  while ((buf = (byte *) malloc(--ram)) == NULL); free(buf);


  byte *ptr = EdWws2812.begin();
  if ( EdWws2812.EdWws2812_Err == 1 ) {
    Serial.print("Err: Za malo RAM na bufor LED WS2812!");
    sprintf_P( txt, PSTR(" Wymagane %d bajtów, teoretycznie dostępne %d"), EdWws2812.EdWws2812_Ram, ram ); Serial.println(txt);
  }
  else if ( EdWws2812.EdWws2812_Err ) {
    sprintf_P( txt, PSTR("Err WS2812 %d %d"), EdWws2812.EdWws2812_Err, ptr ); Serial.println(txt);
  }
  else {
    sprintf_P( txt, PSTR("WS2812 paleta %d-bit %x Zarezerwowoano %d bajtów RAM dla %d led. Teoretycznie zostało %d bajtów"), EdWws2812.EdWws2812_paleta, EdWws2812.EdWws2812_st, EdWws2812.EdWws2812_Ram, L_LED, ram ); Serial.println(txt);
  }


  skanujI2c();

  Serial.println("*** START ***");
}

//================================================================//
void loop() {
  int static krok;
  byte static skok, dir = 1;


  EdWws2812.clr();
  if ( (cfgWs2812 & WS2812_CFG_COLOR_MSK ) == WS2812_CFG_COLOR_218 ) {  // 218 barw
    byte static ofs;
    /*
      uint32_t static t;
      if ( millis() > t ) {
      t = millis() + 2000;
      ofs++;
      }
    */
    for (uint16_t x = 0; x < L_LED; x++ ) {
      EdWws2812.setLed( x + skok, (ofs * L_LED) + x, 0, 0);
    }
  }
  else  {  // 8, 16 i 24 bit
    for (uint16_t x = 0; x < L_LED; x += 6 ) {
      EdWws2812.setLed( x + 0 + skok, krok, 1, 1);
      EdWws2812.setLed( x + 1 + skok, 2, krok, 2);
      EdWws2812.setLed( x + 2 + skok, 3, 3, krok );
      EdWws2812.setLed( x + 3 + skok, maxKrok - krok, 1, 1);
      EdWws2812.setLed( x + 4 + skok, 2, maxKrok - krok, 2);
      EdWws2812.setLed( x + 5 + skok, 3, 3, maxKrok - krok );
    }
  }

  //uint32_t time = millis();
  //Serial.print(">");
  //digitalWrite(9, LOW);

  byte static jas = 30, di = 1;
  if ( (cfgWs2812 & WS2812_CFG_COLOR_MSK ) == WS2812_CFG_COLOR_218 ) EdWws2812.show( jas );
  else EdWws2812.show( JASNOSC_LED );  // Jasność LED dla trybów 8-bit (6 najstarszych bitów)

  //digitalWrite(9, HIGH);
  //time = millis() - time;
  //char txt[50];
  // sprintf_P( txt, PSTR("< %dms"), time ); Serial.println(txt);


  if ( di ) {
    if ( jas < JASNOSC_LED ) jas += 5; else {
      di = 0;
    }
  }
  else {
    if ( jas > 30 ) jas -= 5; else {
      di = 1;
      jas = 30;
    }
  }


  if ( dir ) {
    if ( krok < maxKrok ) krok += 5; else {
      dir = 0;
    }
  }
  else {
    if ( krok > 0 ) krok -= 5; else {
      dir = 1;
      krok = 0;
      skok++;
    }
  }
  if ( skok >= 8 ) skok = 0;


  delay(100);
}
